from .config import DATA_DIR, RESULTS_PATH, N_ITER, BIN_SIZE
from .data_loader import load_and_prepare_data
from .graph_utils import build_ppi_graph
from .analysis import compute_all_proximities
import graph_tool.all as gt
import numpy as np

def main():
    triples, dru_pro, dis_pat = load_and_prepare_data(DATA_DIR)
    g, ids, gene2id, id2gene, all_genes_ppi = build_ppi_graph(DATA_DIR)

    dis_pat = dis_pat[dis_pat.gene_id.isin(all_genes_ppi)].reset_index(drop=True)
    dru_pro = dru_pro[dru_pro.gene_id.isin(all_genes_ppi)].reset_index(drop=True)
    triples = triples.merge(dis_pat[['disease_id', 'pathway_id']].drop_duplicates(), on=['disease_id', 'pathway_id'])
    triples = triples.merge(dru_pro[['drug_id']].drop_duplicates(), on='drug_id')

    print(f"Running proximity analysis on {len(triples)} triples...")

    print("Precomputing all-pairs shortest distances...")
    spls = gt.shortest_distance(g)
    shortest_dict = {i: spls[i] for i in range(g.num_vertices())}

    avg_spls_list = []
    for i, v in enumerate(g.iter_vertices()):
        tmp = spls[v].a
        mask = np.ones(tmp.shape, dtype=bool)
        mask[tmp == 2147483647] = 0
        mask[i] = 0
        if tmp[mask].shape[0]:
            avg_spls_list.append(np.mean(tmp[mask]))

    df_result = compute_all_proximities(triples, dis_pat, dru_pro, g, ids, gene2id, id2gene, shortest_dict, N_ITER, BIN_SIZE)
    df_result.to_csv(RESULTS_PATH, index=False)
    print(f"Results saved to {RESULTS_PATH}")

if __name__ == "__main__":
    main()