import pandas as pd
import tqdm
from .proximity_metrics import get_min_and_avg_spls_A_B, get_proximity_z_score, get_shortest_paths_A_B, get_edged_A_B

def compute_all_proximities(triples, dis_pat, dru_pro, g, ids, gene2id, id2gene, shortest_dict, n_iter, bin_size):
    results = []
    for _, row in tqdm.tqdm(triples.iterrows(), total=len(triples)):
        disease, pathway, drug = row['disease_id'], row['pathway_id'], row['drug_id']
        dis_genes = dis_pat.query("disease_id == @disease and pathway_id == @pathway")['gene_id']
        drug_genes = dru_pro.query("drug_id == @drug")['gene_id']

        dis_genes = [gene2id[g] for g in dis_genes if g in gene2id]
        drug_genes = [gene2id[g] for g in drug_genes if g in gene2id]
        if not dis_genes or not drug_genes:
            continue

        min_spl, avg_spl = get_min_and_avg_spls_A_B(dis_genes, drug_genes, shortest_dict)
        z_score = get_proximity_z_score(g, dis_genes, drug_genes, shortest_dict, n_iter, bin_size, ids, gene2id)
        dispat_closer = get_shortest_paths_A_B(dis_genes, drug_genes, g, id2gene)
        closer_edges = get_edged_A_B(dis_genes, drug_genes, g, id2gene)

        results.append({
            'disease_id': disease,
            'pathway_id': pathway,
            'drug_id': drug,
            'n_dis_pat_genes': len(dis_genes),
            'n_dru_genes': len(drug_genes),
            'min_spl': min_spl,
            'avg_spl': avg_spl,
            'z_score': z_score,
            'disease_pathway_closer_proteins': closer_edges['A_prots'],
            'drug_closer_proteins': closer_edges['B_prots'],
            'dispat_dru_closer_connections': closer_edges['paths'],
            'protein_paths': dispat_closer 
        })
    return pd.DataFrame(results)