
import pandas as pd
import graph_tool.all as gt

def build_ppi_graph(data_dir):
    ppi = pd.read_csv(f'{data_dir}/pro_pro.tsv', sep='\t')
    gen_pro = pd.read_csv(f'{data_dir}/gen_pro.tsv', sep='\t')

    ppi = (
        ppi.merge(gen_pro, left_on='prA', right_on='pro').drop('pro', axis=1).rename(columns={'gen': 'genA'})
           .merge(gen_pro, left_on='prB', right_on='pro').drop('pro', axis=1).rename(columns={'gen': 'genB'})
           [['genA', 'genB']]
           .drop_duplicates()
           .reset_index(drop=True)
    )

    all_genes_ppi = pd.unique(ppi[['genA', 'genB']].values.ravel())

    g = gt.Graph(directed=False)
    ids = g.add_edge_list(ppi[['genA', 'genB']].values, hashed=True)

    g.vertex_properties['ids'] = ids
    gene2id = {ids[x]: x for x in g.iter_vertices()}
    id2gene = {v: k for k, v in gene2id.items()}

    return g, ids, gene2id, id2gene, all_genes_ppi
